﻿/*
 * Copyright (c) 2011 Achim 'ahzf' Friedland <achim@ahzf.de>
 * This file is part of graph-database.org <http://www.github.com/Fellmann/graph-database.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * You may obtain a copy of the License at
 *     http://www.gnu.org/licenses/gpl.html
 *     
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 */

#region Usings

using System;
using System.Linq;
using System.Threading;

using de.ahzf.Blueprints.PropertyGraphs.InMemory.Mutable;
using de.ahzf.fellmann.REST;

using de.ahzf.Illias.Commons;
using de.ahzf.Hermod;
using de.ahzf.Hermod.Datastructures;
using de.ahzf.Hermod.HTTP;
using de.ahzf.Blueprints.PropertyGraphs;

#endregion

namespace Runner
{
    
    public class Program
    {

        #region Const

        public const String _Name         = "Name";
        public const String _Description  = "Description";
        public const String _URI          = "Uri";

        public const String is_a          = "is_a";
        public const String is_edition_of = "is_edition_of";
        public const String written_in    = "written_in";
        public const String licensed      = "licensed";

        #endregion

        public static void Main(String[] args)
        {

            var _PropertyGraph = new PropertyGraph();
            var _GraphServer   = new GraphServer(_PropertyGraph, IPv4Address.Any, IPPort.HTTP, Autostart: true);
            Console.WriteLine(_GraphServer);


            #region GraphDB vertices

            var GraphDBs                = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "GraphDBs").
                                                                          SetProperty(_Description, "A datastructure implementing a property graph."));
            _GraphServer.AddToIndex(GraphDBs);


            var AllegroGraph            = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "AllegroGraph").
                                                                          SetProperty(_URI,         "http://www.franz.com/agraph"));

            var DEX                     = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "DEX").
                                                                          SetProperty(_URI,         "http://sparsity-technologies.com"));

            var Filament                = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Filament").
                                                                          SetProperty(_URI,         "http://filament.sourceforge.net"));

            var HyperGraphDB            = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "HyperGraphDB").
                                                                          SetProperty(_URI,         "http://www.hypergraphdb.org"));

            var InfiniteGraph           = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "InfiniteGraph").
                                                                          SetProperty(_URI,         "http://infinitegraph.com"));

            var InfoGrid                = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "InfoGrid").
                                                                          SetProperty(_URI,         "http://infogrid.org"));


            var Neo4j                   = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Neo4j").
                                                                          SetProperty(_URI,         "http://www.neo4j.org"));

            var Neo4jCE                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Neo4j Community Edition"));

            var Neo4jAE                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Neo4j Advanced Edition"));

            var Neo4jEE                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Neo4j Enterprise Edition"));


            var OrientDB                = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "OrientDB").
                                                                          SetProperty(_URI,         "http://www.orientechnologies.com"));

            var SonesGraphDB            = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Sones GraphDB").
                                                                          SetProperty(_URI,         "http://www.sones.de"));

            var VertexDB                = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "VertexDB").
                                                                          SetProperty(_URI,         "http://www.dekorte.com/projects/opensource/vertexdb"));

            var CloudGraph              = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "CloudGraph").
                                                                          SetProperty(_Description, "A disk- and memory-based, fully transactional .NET graph database that uses graphs and key/value pairs to store data.").
                                                                          SetProperty(_URI,         "http://www.dekorte.com/projects/opensource/vertexdb"));

            var Horton                  = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Horton").
                                                                          SetProperty(_Description, "A graph database from Microsoft Research Extreme Computing Group (XCG) based on the cloud programming infrastructure Orleans.").
                                                                          SetProperty(_URI,         "http://www.dekorte.com/projects/opensource/vertexdb"));

            _PropertyGraph.AddEdge(AllegroGraph,  is_a, GraphDBs);
            _PropertyGraph.AddEdge(DEX,           is_a, GraphDBs);
            _PropertyGraph.AddEdge(Filament,      is_a, GraphDBs);
            _PropertyGraph.AddEdge(HyperGraphDB,  is_a, GraphDBs);
            _PropertyGraph.AddEdge(InfiniteGraph, is_a, GraphDBs);
            _PropertyGraph.AddEdge(InfoGrid,      is_a, GraphDBs);

            _PropertyGraph.AddEdge(Neo4j,         is_a,          GraphDBs);
            _PropertyGraph.AddEdge(Neo4jCE,       is_edition_of, Neo4j);
            _PropertyGraph.AddEdge(Neo4jAE,       is_edition_of, Neo4j);
            _PropertyGraph.AddEdge(Neo4jEE,       is_edition_of, Neo4j);

            _PropertyGraph.AddEdge(OrientDB,      is_a, GraphDBs);
            _PropertyGraph.AddEdge(SonesGraphDB,  is_a, GraphDBs);
            _PropertyGraph.AddEdge(VertexDB,      is_a, GraphDBs);
            _PropertyGraph.AddEdge(CloudGraph,    is_a, GraphDBs);
            _PropertyGraph.AddEdge(Horton,        is_a, GraphDBs);

            #endregion

            #region Graph Processing vertices

            var GraphProcessing         = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "GraphProcessing"));
            _GraphServer.AddToIndex(GraphProcessing);

            var SignalAndCollect        = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "SignalAndCollect").
                                                                          SetProperty(_Description, "Signal/Collect is a framework for parallel graph processing.").
                                                                          SetProperty(_URI,         "http://graphlab.org"));

            var GraphLab                = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "GraphLab").
                                                                          SetProperty(_Description, "A new Parallel Framework for Machine Learning.").
                                                                          SetProperty(_URI,         "http://graphlab.org"));

            var Giraph                  = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Giraph").
                                                                          SetProperty(_Description, "A Java BSP implementation that runs on existing Hadoop clusters and provides fault tolerance for both the workers and the master using ZooKeeper. Giraph came out of Yahoo! and is now an Apache Incubator project.").
                                                                          SetProperty(_URI,         "http://incubator.apache.org/giraph"));

            var Phoebus                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Phoebus").
                                                                          SetProperty(_Description, "An Erlang BSP implementation that can use HDFS for storage.").
                                                                          SetProperty(_URI,         "http://github.com/xslogic/phoebus"));

            var GoldenOrb               = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "GoldenOrb").
                                                                          SetProperty(_Description, "A Java BSP implementation ontop of Hadoop.").
                                                                          SetProperty(_URI,         "http://www.goldenorbos.org"));

            var Spark                   = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Spark").
                                                                          SetProperty(_Description, "A Scala framework from UC Berkeley that aims to balance efficiency and fault tolerance by providing immutable, distributed, in-memory collections that remember how they were created and can repair themselves automatically.").
                                                                          SetProperty(_URI,         "http://graphlab.org"));

            var Bagel                   = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Bagel").
                                                                          SetProperty(_Description, "A BSP implementation on top of Spark.").
                                                                          SetProperty(_URI,         "http://github.com/mesos/spark/wiki/Bagel-Programming-Guide"));

            var Piccolo                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Piccolo").
                                                                          SetProperty(_Description, "A C++ system from NYU that provides an in-memory shared table with checkpointing for fault tolerance and accumulation for concurrency.").
                                                                          SetProperty(_URI,         "http://piccolo.news.cs.nyu.edu"));

            var HaLoop                  = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "HaLoop").
                                                                          SetProperty(_Description, "A version of Hadoop from the University of Washington that has been modified and optimized to support iteration.").
                                                                          SetProperty(_URI,         "http://code.google.com/p/haloop"));

            var Trinity                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Trinity").
                                                                          SetProperty(_Description, "...").
                                                                          SetProperty(_URI,         "http://research.microsoft.com/en-us/projects/trinity"));

            var Twister                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Twister").
                                                                          SetProperty(_Description, "An iterative MapReduce runtime from Indiana University.").
                                                                          SetProperty(_URI,         "http://www.iterativemapreduce.org"));

            var Ciel                    = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Ciel").
                                                                          SetProperty(_Description, "A Python system that, along with the Skywriting language, provides a variety of data flow operators and handles fault tolerance in a similar way as Spark does, although without the performance advantage of keeping data in memory across jobs.").
                                                                          SetProperty(_URI,         "http://www.cl.cam.ac.uk/research/srg/netos/ciel"));

            var Pegasus                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name,        "Pegasus").
                                                                          SetProperty(_Description, "PEGASUS is a Peta-scale graph mining system, fully written in Java. It runs in parallel, distributed manner on top of Hadoop. Hadoop is a cloud computing platfrom, as well as an open source implementation of MapReduce framework which was originally designed for web-scale data processing by Google.").
                                                                          SetProperty(_URI,         "http://www.cs.cmu.edu/~pegasus/index.htm"));


            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, SignalAndCollect);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, GraphLab);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, Giraph);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, Phoebus);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, GoldenOrb);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, Spark);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, Bagel);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, Piccolo);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, HaLoop);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, Trinity);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, Twister);
            _PropertyGraph.AddHyperEdge(is_a, GraphProcessing, Pegasus);

            #endregion

            #region Programming Language vertices

            var ProgrammingLanguages    = _PropertyGraph.AddVertex(v => v.SetProperty(_Name, "ProgrammingLanguages"));

            var Java                    = _PropertyGraph.AddVertex(v => v.SetProperty(_Name, "Java"));


            _PropertyGraph.AddEdge(Java, ProgrammingLanguages, Label: "is_a");

            _PropertyGraph.AddEdge(Neo4j,    Java, Label: written_in);
            _PropertyGraph.AddEdge(OrientDB, Java, Label: written_in);

            #endregion

            #region License vertices

            var Licenses                = _PropertyGraph.AddVertex(v => v.SetProperty(_Name, "Licenses"));

            var Apache2                 = _PropertyGraph.AddVertex(v => v.SetProperty(_Name, "Apache 2.0"));
            var AGPLv3                  = _PropertyGraph.AddVertex(v => v.SetProperty(_Name, "AGPLv3"));
            var GPLv3                   = _PropertyGraph.AddVertex(v => v.SetProperty(_Name, "GPLv3"));

            _PropertyGraph.AddEdge(Apache2,          Licenses, Label: is_a);
            _PropertyGraph.AddEdge(AGPLv3,           Licenses, Label: is_a);
            _PropertyGraph.AddEdge(GPLv3,            Licenses, Label: is_a);

            _PropertyGraph.AddEdge(Neo4jCE,          GPLv3,  licensed);
            _PropertyGraph.AddEdge(Neo4jAE,          AGPLv3, licensed);
            _PropertyGraph.AddEdge(Neo4jEE,          AGPLv3, licensed);

            _PropertyGraph.AddEdge(OrientDB,         Apache2, licensed);
            _PropertyGraph.AddEdge(SignalAndCollect, Apache2, licensed);

            #endregion




            var a1 = _PropertyGraph.VertexById(2);

            // ------------------------------------------------------

            var _IGenericPropertyGraph = _PropertyGraph as IGenericPropertyGraph<UInt64, Int64, String, String, Object,
                                                                                 UInt64, Int64, String, String, Object,
                                                                                 UInt64, Int64, String, String, Object,
                                                                                 UInt64, Int64, String, String, Object>;
            var a2 = _IGenericPropertyGraph.VertexById(1);

            // ------------------------------------------------------

            var _IPropertyGraph = _PropertyGraph as IPropertyGraph;
            var a3 = _IPropertyGraph.VertexById(1);
            var b3 = _IPropertyGraph.Vertices();

            // ------------------------------------------------------

            Thread.Sleep(500);

            //var _clientA  = new HTTPClient(IPv4Address.Parse("127.0.0.1"), IPPort.HTTP);
            //var _requestA = _clientA.GET("/AllGraphDBs").
            //                         SetProtocolVersion(HTTPVersion.HTTP_1_1).
            //                         SetUserAgent("Hermod HTTP Client v0.1").
            //                         SetConnection("keep-alive").
            //                         AddAccept(HTTPContentType.JSON_UTF8, 1);

            //_clientA.Execute(_requestA, response => Console.WriteLine(response.Content.ToUTF8String()));

            //Console.ReadLine();

            var _clientB  = new HTTPClient(IPv4Address.Parse("127.0.0.1"), IPPort.HTTP);
            var _requestB = _clientB.GET("/AccountId/RepositoryId/TransactionId/GraphId/VerticesById?Id=2&Id=3").
                                     SetProtocolVersion(HTTPVersion.HTTP_1_1).
                                     SetUserAgent("Hermod HTTP Client v0.1").
                                     SetConnection("keep-alive").
                                     AddAccept(HTTPContentType.JSON_UTF8, 1);

            _clientB.Execute(_requestB, response => Console.WriteLine(response.Content.ToUTF8String()));

            Console.ReadLine();
            Console.WriteLine("done!");

        }

    }

}
